/******************************************************************************
 * 
 * Copyright 2012 - 2014 Vektor Software.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *****************************************************************************/


package com.vektorsoft.demux.android.core.app;

import com.vektorsoft.demux.core.mva.DMXAdapter;
import com.vektorsoft.demux.core.mva.DMXEventRegistrationHandler;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * <p>
 * Base class for implementing event handlers in Android. This class is parametrized, where type parameter 
 * is the type of the event listener to be implemented. Sample usage of this class is shown bellow:
 * </p>
 * <p>
 *  <code>
 *      <pre>
 *          public class SomeEventHandler &lt;View.OnClickListener &gt; implements View.OnClickListener {
 *              ........................
 *      </pre>
 *  </code>
 * </p>
 *
 * @param <T> type of event listener which will handle this event
 * @author Vladimir Djurovic
 */
public abstract class  AndroidEventHandler<T> implements DMXEventRegistrationHandler<T> {
    
    /** Adapter instance. */
    protected DMXAdapter adapter;
    
    /** List of component IDs to which this handler will be registered. */
    private final List<String> componentIds;
    
    
    /**
     * Creates new instance.
     * 
     * @param adapter adapter instance
     */
    protected AndroidEventHandler(DMXAdapter adapter){
        this.adapter = adapter;
        componentIds = new ArrayList<String>();
    }

    @Override
    public String[] getComponentIds() {
        return componentIds.toArray(new String[componentIds.size()]);
    }

    /**
     * Set components IDs.
     * 
     * @param componentIds  component IDs as string
     */
    public void setComponentIds(String... componentIds) {
        this.componentIds.addAll(Arrays.asList(componentIds));
    }
    
    /**
     * Add component ID to existing list.
     * 
     * @param id component ID as string
     */
    public void addComponentId(String id){
        componentIds.add(id);
    }
    
    /**
     * Add component ID to existing list.
     * 
     * @param id  component ID as integer
     */
    public void addComponentId(int id){
        componentIds.add(Integer.toString(id));
    }
    

}
