/******************************************************************************
 * 
 * Copyright 2012 - 2014 Vektor Software.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *****************************************************************************/


package com.vektorsoft.demux.android.core.resources;

import android.content.Context;
import android.content.res.Configuration;
import com.vektorsoft.demux.core.extension.DMXExtensionCallback;
import com.vektorsoft.demux.core.resources.DMXResourceManager;
import com.vektorsoft.demux.core.resources.LocaleObservable;
import com.vektorsoft.demux.core.resources.ResourceHandlerType;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Locale;
import java.util.Observer;

/**
 *
 * @author Vladimir Djurovic
 */
public class DMXAndroidResourceManager implements DMXResourceManager{
    
    /** List of locales supported by application. */
    protected List<Locale> supportedLocales;
    
    /** Application's current locale. */
    protected Locale currentLocale;
    
    /**
     * Resource handler for this resource manager.
     * 
     */
    protected DMXAndroidResourceHandler resourceHandler;
    
    /**
     * App context.
     */
    protected Context context;
   
    
    /** Observable object for notifying observers about locale change. */
    private LocaleObservable observable;
    
    /**
     * Creates new instance.
     * 
     * @param context app context
     */
    public DMXAndroidResourceManager(Context context){
        this.context = context;
        currentLocale = Locale.getDefault();
        supportedLocales = new ArrayList<Locale>();
        observable = new LocaleObservable();
        resourceHandler = new DMXAndroidResourceHandler(context);
        resourceHandler.setCurrentLocale(currentLocale);
    }

    @Override
    public void setCurrentLocale(Locale locale) {
         if(locale == null){
            throw new IllegalArgumentException("Locale cannot be null");
        }
        this.currentLocale = locale;
        resourceHandler.setCurrentLocale(locale);
        // change application locale
        Configuration config = new Configuration();
        config.locale = this.currentLocale;
        context.getResources().updateConfiguration(config, context.getResources().getDisplayMetrics());
        observable.onLocaleChanged();
        observable.notifyObservers();
    }

    @Override
    public Locale getCurrentLocale() {
        return currentLocale;
    }

    @Override
    public void setSupportedLocales(Locale[] locales) {
         if(locales == null){
            throw new IllegalArgumentException("Supported locales cannot be null");
        }
        supportedLocales.clear();
        supportedLocales.addAll(Arrays.asList(locales));
    }

    @Override
    public Locale[] getSupportedLocales() {
        return supportedLocales.toArray(new Locale[supportedLocales.size()]);
    }

    @Override
    public void addSupportedLocale(Locale locale) {
         if(locale == null){
            throw new IllegalArgumentException("Supported locale cannot be null");
        }
        supportedLocales.add(locale);
    }

    @Override
    public void clearSupportedLocales() {
        supportedLocales.clear();
    }

    @Override
    public DMXAndroidResourceHandler getResourceHandler(ResourceHandlerType type) {
        if(type == ResourceHandlerType.ANDROID){
            return  resourceHandler;
        } else {
            throw new IllegalArgumentException("Unsupported resource handler type. Use ResourceHandlerType.ANDROID.");
        }
    }

    @Override
    public void addLocaleChangeObserver(Observer observer) {
        observable.addObserver(observer);
    }

    @Override
    public void registerCallbackResources(DMXExtensionCallback callback) {
        resourceHandler.loadCallbackResources(callback);
    }

}
