/******************************************************************************
 * 
 * Copyright 2012 - 2014 Vektor Software.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *****************************************************************************/


package com.vektorsoft.demux.core.app;

import com.vektorsoft.demux.core.extension.DMXExtensionConfig;
import com.vektorsoft.demux.core.extension.DefaultExtensionConfig;
import com.vektorsoft.demux.core.mva.DMXAdapter;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.util.tracker.ServiceTracker;

/**
 * <p>
 *  Basic activator class which provides support for extensions through fragment bundles. Bundle which are intended for
 * extension by callbacks should use implementation of this class as activator, because it provides consistent process for
 * registering view, controllers, extension callbacks and also provides a way to customize bundle startup sequence. This class 
 * provides the following startup process:
 * </p>
 * <p>
 *  <ol>
 *  <li>On startup, it will try to find an instance of {@link DMXExtensionConfig}. This is usually provided by a fragment bundle
 * which contains extension callbacks for this bundle (it's host). It will look for a class {@code BundleExtensionConfig} in {@code ext}
 * subpackage of current package. For example, if bundle activator is in package {@code com.mypkg}, it will look for class 
 * {@code com.mypkg.ext.BundleExtensionConfig}.
 * </li>
 *  <li>
 *   Next step is to register service tracker of for {@link DMXAdapter} service. This tracker will use extension configuration (if found) to register any 
 * extension callbacks specified.
 *  </li>
 *  <li>
 *  Finally, it will invoke method from sublcass which can add custom functionality to bundle activator.
 *  </li>
 *  </ol>
 * </p>
 *
 * @author Vladimir Djurovic
 */
public abstract class DMXAbstractActivator implements BundleActivator{
    
    /** Name of the extension config class relative to package of this activator. */
    private static final String EXTENSION_CONFIG_CLASS_NAME = ".ext.BundleExtensionConfig";
    
    /** Extension configuration. */
    protected DMXExtensionConfig extensionConfig;
    
    /** Customizer for {@code DMXAdapter} service tracker. */
    private DMXAdapterTrackerCustomizer adapterCustomizer;
    
    /** Service tracker for {@code DMXAdapter} service. */
    private ServiceTracker<DMXAdapter, DMXAdapter> adapterTracker;
    
    /**
     * This method is invoked at the end of {@link #start(org.osgi.framework.BundleContext) } method. It is used
     * as a hook for adding custom functionality to bundle activator. Subclasses should implement this method if they want 
     * to add custom behavior to activator start method.
     * 
     * @param ctx bundle context
     */
    protected abstract void onBundleStart(BundleContext ctx);
    
    /**
     * This method is invoked at the end of {@link #stop(org.osgi.framework.BundleContext) } method. It is used
     * as a hook for adding custom functionality to bundle activator stopping. Subclasses should implement this method if they want 
     * to add custom behavior to activator stop method.
     * 
     * @param ctx bundle context
     */
    protected abstract void onBundleStop(BundleContext ctx);

    /**
     * Subclasses should implement this method to provide custom {@link DMXAdapterTrackerHandler} instance. This 
     * instance is used to customize tracker method for adding, modifying and removing the service.
     * 
     * @return tracker handler instance.
     */
    public abstract DMXAdapterTrackerHandler getHandler();
    

    @Override
    public final void start(BundleContext bc) throws Exception {
        
        getBundleExtensionConfig();
        // register service tracker for adapter service
        adapterCustomizer = new DMXAdapterTrackerCustomizer(bc, extensionConfig);
        adapterCustomizer.setHandler(getHandler());
        adapterTracker = new ServiceTracker<DMXAdapter, DMXAdapter>(bc, DMXAdapter.class, adapterCustomizer);
        adapterTracker.open();
        
        onBundleStart(bc);
      
    }

    @Override
    public final void stop(BundleContext bc) throws Exception {
        if(adapterTracker != null){
            adapterTracker.close();
        }
        onBundleStop(bc);
    }
    
    /**
     * Tried to find an instance of {@link DMXExtensionConfig}. This would usually be supplied by a fragment bundle. 
     * This method will look for a class named {@code ext.BundleExtensionConfig} relative to activator's package. If this
     * class does not exist, it will set an instance of {@link DefaultExtensionConfig} as extension configuration.
     */
    private void getBundleExtensionConfig(){
        Class clazz = this.getClass();
        StringBuilder sb = new StringBuilder(clazz.getPackage().getName());
        sb.append(EXTENSION_CONFIG_CLASS_NAME);
        
        try{
            Class extClass = Class.forName(sb.toString(), true, clazz.getClassLoader());
            extensionConfig = (DMXExtensionConfig)extClass.newInstance();
        } catch(ClassNotFoundException ex){
            extensionConfig = new DefaultExtensionConfig();
        } catch(InstantiationException iex){
            extensionConfig = new DefaultExtensionConfig();
        } catch(IllegalAccessException iaex){
            extensionConfig = new DefaultExtensionConfig();
        }
    }
    

}
