/******************************************************************************
 * 
 * Copyright 2012 - 2014 Vektor Software.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *****************************************************************************/

package com.vektorsoft.demux.core.extension;

import java.util.Map;

/**
 * Defines interface for extension callbacks used to customize behavior of views and 
 * controllers.
 *
 * @author Vladimir Djurovic
 */
public interface DMXExtensionCallback {
    
    /**
     * Returns a type of this extension. It could be either an extension for
     * view or controller.
     * 
     * @return extension type enumeration
     */
    DMXExtensionType getExtensionType();

    /**
     * Returns the mappings of views or controllers to which this
     * extension will apply. One callback can be used in multiple views 
     * or controllers.
     * 
     * @return class name
     */
    String[] getMappings();
    
    /**
     * Returns data that this callback wants to register with the model. These
     * are registered in addition to original data from view or controller, and
     * are handled in the same way as original data.
     * 
     * @return data mapping
     */
    Map<String, Object> getCallbackData();
    
    /**
     * Updates model which this callback has registered with values currently set 
     * in model. This method should be called before extendable is executed.
     * 
     * @param dataMap data map from model
     */
    void setDataValuesFromModel(Map<String, Object> dataMap);
    
    /**
     * <p>
     * Returns a list of names of resources bundles associated with this extension callback.
     * Values specified in bundles will override values set in extendable controller or view. The way
     * to do this is to specify name of each property from {@link DMXExtendable} which is being overriden
     * as a key, and set new property value as a value.
     * </p>
     * <p>
     *  For example, let's say that {@code DMXExtendable} specifies the following properties:
     *  <pre>
     *      <code>
     *          ext.prop1 = value 1
     *          ext.prop2 = value 2
     *          ext.prop3 = value 3
     *      </code>
     *  </pre>
     * To override these values in extension callback, create a resource bundle which contains values to be overriden.
     * 
     * <pre>
     *      <code>
     *          ext.prop1 = Override value 1
     *          ext.prop2 = Overridevalue 2
     *      </code>
     *  </pre>
     * 
     *  and then return name of this bundle as a return from this method.
     * 
     * <pre>
     *      <code>
     *          String[] getResourceBundles(){
     *              return new String[]{"my.bundle};
     *          }
     *      </code>
     *  </pre>
     * </p>
     * 
     * <p>
     *  For platforms which do not use resource bundle to manage resources, such as Android, properties should map parent 
     * values to values which will override them. For example:
     * <pre>
     *      <code>
     *          original_string_name = callback_string_name
     *          original_drawable_name = callback_drawable_name
     *      </code>
     *  </pre>
     * 
     * Note that, due to the packaging of Android applications, these values should not be specified in properties files, but rather 
     * in implementations such as {@code ListResourceBundle}.
     * 
     * </p>
     * 
     * @return array of resource bundles names
     */
    String[] getResourceBundles();
    
}
