/******************************************************************************
 * 
 * Copyright 2012 - 2014 Vektor Software.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *****************************************************************************/


package com.vektorsoft.demux.desktop.jfx.app.gui;

import com.vektorsoft.demux.core.mva.DMXEventRegistrationHandler;
import com.vektorsoft.demux.desktop.gui.DMXToolbarService;
import com.vektorsoft.demux.desktop.jfx.gui.JFXEventHandler;
import javafx.application.Platform;
import javafx.geometry.Insets;
import javafx.scene.Node;
import javafx.scene.control.ToolBar;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.stage.Stage;

/**
 * Implementation of {@link DMXToolbarService}. It provides a way to add toolbars 
 * to top of the main window, right under main menu bar.
 *
 * @author Vladimir Djurovic
 */
public class JFXToolbarServiceImpl implements DMXToolbarService<ToolBar> {
    
    /**
     * Default insets value for main toolbar.
     */
    private static final int DEFAULT_TOOLBAR_INSETS = 3;
    
    /**
     * Default spacing between individual toolbars.
     */
    private static final int DEFAULT_SPACING = 5;
    
    /** Container for housing menu bar and toolbars. Menubar should be on top. */
    private final  VBox topContainer;
    
    /** Container for housing toolbars. */
    private HBox toolbarContainer;
    

    /**
     * Creates new instance.
     * 
     * @param stage main window stage
     */
    public JFXToolbarServiceImpl(Stage stage){
        this.topContainer = ((VBox)((BorderPane)stage.getScene().getRoot()).getTop());
        init();
    }
    
    /**
     * Initializes basic GUI layout.
     */
    private void init(){
        Platform.runLater(new Runnable() {

            @Override
            public void run() {
                toolbarContainer = new HBox();
                toolbarContainer.setPadding(new Insets(DEFAULT_TOOLBAR_INSETS));
                toolbarContainer.setSpacing(DEFAULT_SPACING);
                // we add toolbar container to 2nd index, under main menu bar
                topContainer.getChildren().add(1, toolbarContainer);
            }
        });
    }
    
    @Override
    public void addToolbar(final ToolBar toolbar) {
        Platform.runLater(new Runnable() {

            @Override
            public void run() {
                toolbarContainer.getChildren().add(toolbar);
            }
        });
    }

    @Override
    public void insertToolbarAtIndex(final int index, final ToolBar toolbar) {
        Platform.runLater(new Runnable() {

            @Override
            public void run() {
                toolbarContainer.getChildren().add(index, toolbar);
            }
        });
    }

    @Override
    public void setToolbarStyle(final String style) {
        Platform.runLater(new Runnable() {

            @Override
            public void run() {
                toolbarContainer.setStyle(style);
            }
        });
    }

    @Override
    public void registerEventHandlerForId(final String id, final DMXEventRegistrationHandler handler) {
        if(handler instanceof JFXEventHandler){
            Platform.runLater(new Runnable() {

                @Override
                public void run() {
                    JFXEventHandler jhandler = (JFXEventHandler)handler;
                    Node n = toolbarContainer.lookup("#" + id);
                    System.out.println("Toolbar: looked up node " + id);
                    if(n != null){
                        n.addEventHandler(jhandler.getEventType(), jhandler);
                    }
                }
            });
            
        } else {
            throw new IllegalArgumentException("Invalid event handler type, required com.vektorsoft.demux.desktop.app.JFXEventHandler");
        }
    }
    

}
